# Label row constants
LABELS = "labels"
OBJECTS = "objects"
CLASSIFICATIONS = "classifications"
OBJECT_HASH = "objectHash"
CLASSIFICATION_HASH = "classificationHash"
OBJECT_ANSWERS = "object_answers"
CLASSIFICATION_ANSWERS = "classification_answers"

# Data types
VIDEO = "video"
IMG_GROUP = "img_group"

# Labeling algorithm names
INTERPOLATION = "interpolation"
